/*
 * Copyright (c) 1998, 2002 Sandro Sigala.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef GAMEENGINE_H
#define GAMEENGINE_H

#include <vector>

#include "fxzacman.h"

enum Sound {
	Sound_Died,
	Sound_Got_Pill,
	Sound_Got_Potion,
	Sound_Potion_Timeout,
	Sound_Saw_Monster,
	Sound_Killed_Monster,
	Sound_Cannot_Exit,
};

enum Status {
	Status_Playing,
	Status_Finished,
	Status_Died
};

enum Skill {
	Skill_Beginner,
	Skill_Intermediate,
	Skill_Advanced,
	Skill_Expert
};

enum Direction {
	North,
 	South,
  	East,
   	West
};

struct Monster {
	int xpos, ypos;
	Sprite sprite;
	Direction dir;
	bool saw;
	bool dead;

	Monster(int x, int y, Sprite s)
		: xpos(x), ypos(y), sprite(s), dir(North) {
  			saw = false; dead = false;
     	}
};

struct GameEngine {
private:
	Sprite *map;		// The map array
	vector<Monster *> monsters;

	Sprite& ref(int x, int y) {
		return map[y*map_width + x];
	}

	void deleteMonsters();
	void addMonster(int x, int y);
	void killMonster(Monster *mp);
	void updateMonsters();
	bool lookMonster(Monster *mp);
	Direction monsterFindDirection(Monster *mp, Direction exclude);
	void checkPosition(int x, int y);
	void createMap(int x, int y);
	void playSound(Sound snd);

public:
	Skill game_skill;
	Direction dir;		// The orientation of Zacman
	int map_width, map_height;
	int xpos, ypos;
	int num_pills;
	int game_time;
	int potion_time;
	int balloon_time;
	int sounds;
 	int paused;
	Status status;
	Sprite balloon;
	Sprite sprite_wall;	// The sprite used for walls
	Sprite sprite_back;	// The sprite used for non-walls

	GameEngine();
	~GameEngine();

	Sprite xy(int x, int y) {
		if (x < 0 || y < 0 || x >= map_width || y >= map_height)
			return sprite_wall;
		return map[y*map_width + x];
	}
	Monster *findMonster(int x, int y);

	void newGame(Skill skill);
	void goUp();
	void goDown();
	void goLeft();
	void goRight();
	void timer(int msecs);
};

#endif // !GAMEENGINE_H

